function [L,U,p]=lu(A,type)
% lu   permuted LU factorizations of an amat object
% -- [L,U,P]=lu(A)
% -- [L,U,P]=lu(A,'amat')
%      returns unit lower triangular amat L, upper triangular amat U, and 
%      permutation amat P so that 
%         P(k)*A(k) = L(k)*U(k) for all k in 1:A.N
% -- [L,U,p]=lu(A,'vector')
%      returns unit lower triangular amat L, upper triangular amat U, and 
%      permutation index matrice p so that 
%         P(k)*A(k) = L(k)*U(k) for all k in 1:A.N
%      where the k-th permutation matrix Pk (=P(k)) is defined by
%          Pk(i,p(k,i))=1 for all i in 1:A.nr
%      One can used the fc_amat.permind2amat function for building the 
%      permutation amat P:
%          P=fc_amat.permind2amat(p);
%
%      See also: fc_amat.permind2amat, amat.chol
%                class amat
%
  assert( isamat(A) )
  if nargin==1,type='amat';end
  assert( ismember( lower(type),{'matrix','vector','amat'}) )
  
  [N,m,n]=size(A);
  assert( m==n )
  if  ~fc_tools.comp.isOctave() && verLessThan('matlab','9.1.0') % 2016b
    Conv=@(L,s) repmat(L,[1,1,s]);
  else
    Conv=@(L,s) L;
  end
  
  L=fc_amat.eye(N,m);
  U=A;
  %P=fc_amat.eye(N,m);
  p=repmat(1:m,N,1);
  K=[1:N]';
  Subs=@(I,J) sub2ind([N,m,n],K,I,J);
  %J=ones(1,N);
  II=repmat([1:m],N,1);
  for i=1:m-1
    % -> permutations
    [~,idx]=max(abs(U.values(:,i:end,i)),[],2);
    mu=idx+i-1;
    for j=1:i-1
      idx1=Subs(mu,II(:,j));idx2=Subs(II(:,i),II(:,j));
      IdL=[idx1;idx2];IdR=[idx2;idx1];
      L.values(IdL)=L.values(IdR);
      %P.values(IdL)=P.values(IdR);
    end
    for j=i:m
      idx1=Subs(mu,II(:,j));idx2=Subs(II(:,i),II(:,j));
      IdL=[idx1;idx2];IdR=[idx2;idx1];
      U.values(IdL)=U.values(IdR);
      %P.values(IdL)=P.values(IdR);
    end
    idx1=sub2ind([N,m],K,mu);idx2=sub2ind([N,m],K,II(:,i));
    p([idx1;idx2])=p([idx2;idx1]);
    % <- permutations
    kIdx=abs(U.values(:,i,i))>1e-15;
    for j=i+1:m
      L.values(kIdx,j,i)=U.values(kIdx,j,i)./U.values(kIdx,i,i);
      U.values(kIdx,j,i:end)=U.values(kIdx,j,i:end)-Conv(L.values(kIdx,j,i),m-i+1).*U.values(kIdx,i,i:end);
    end
  end
  if strcmp(type,'amat'),  p=fc_amat.permind2amat(p); end
  if nargout==1, L={L,U,p};end
end
